@@ -21,8 +21,8 @@ Information about setting up your Google OAUTH2 credentials:
 For libcloud, there are two basic methods for authenticating to Google using
 OAUTH2: Service Accounts and Client IDs for Installed Applications.
 
-Both are initially set up from the Cloud Console_
-_Console: https://cloud.google.com/console
+Both are initially set up from the Cloud Console Console -
+https://cloud.google.com/console
 
 Setting up Service Account authentication (note that you need the PyCrypto
 package installed to use this):
@@ -38,9 +38,7 @@ package installed to use this):
 - Optionally, you may choose to Generate a PKCS12 key from the Console.
   It needs to be converted to the PEM format.  Please note, the PKCS12 format
   is deprecated and may be removed in a future release.
-  - Convert the key using OpenSSL (the default password is 'notasecret'):
-    ``openssl pkcs12 -in YOURPRIVKEY.p12 -nodes -nocerts
-    -passin pass:notasecret | openssl rsa -out PRIV.pem``
+  - Convert the key using OpenSSL (the default password is 'notasecret').
   - Move the .pem file to a safe location.
 - To Authenticate, you will need to pass the Service Account's "Email
   address" in as the user_id and the path to the .pem file as the key.
@@ -66,6 +64,7 @@ Setting up Installed Application authentication:
 
 Please remember to secure your keys and access tokens.
 """
+
 from __future__ import with_statement
 
 try:
@@ -77,7 +76,6 @@ import base64
 import errno
 import time
 import datetime
-import logging
 import os
 import socket
 import sys
@@ -101,22 +99,30 @@ except ImportError:
     RSA = None
     PKCS1_v1_5 = None
 
-TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
+UTC_TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
 
 
-def _is_gce():
-    http_code, http_reason, body = _get_gce_metadata()
-    if http_code == httplib.OK and body:
-        return True
-    return False
+def _utcnow():
+    """
+    Mocked in libcloud.test.common.google.GoogleTestCase.
+    """
+    return datetime.datetime.utcnow()
+
+
+def _utc_timestamp(datetime_obj):
+    return datetime_obj.strftime(UTC_TIMESTAMP_FORMAT)
+
+
+def _from_utc_timestamp(timestamp):
+    return datetime.datetime.strptime(timestamp, UTC_TIMESTAMP_FORMAT)
 
 
 def _get_gce_metadata(path=''):
     try:
-        url = "http://metadata/computeMetadata/v1/" + path.lstrip('/')
+        url = 'http://metadata/computeMetadata/v1/' + path.lstrip('/')
         headers = {'Metadata-Flavor': 'Google'}
         response = get_response_object(url, headers=headers)
-        return response.status, "", response.body
+        return response.status, '', response.body
     except Exception as e:
         return -1, str(e), None
 
@@ -154,7 +160,7 @@ class ResourceNotFoundError(GoogleBaseError):
                 "error may be an authentication issue. " \
                 "Please  ensure your auth credentials match " \
                 "your project. "
-        super(GoogleBaseError, self).__init__(value, http_code, driver)
+        super(ResourceNotFoundError, self).__init__(value, http_code, driver)
 
 
 class QuotaExceededError(GoogleBaseError):
@@ -328,9 +334,6 @@ class GoogleBaseAuthConnection(ConnectionUserAndKey):
 
         super(GoogleBaseAuthConnection, self).__init__(user_id, key, **kwargs)
 
-    def _now(self):
-        return datetime.datetime.utcnow()
-
     def add_default_headers(self, headers):
         headers['Content-Type'] = "application/x-www-form-urlencoded"
         headers['Host'] = self.host
@@ -348,18 +351,17 @@ class GoogleBaseAuthConnection(ConnectionUserAndKey):
         :rtype:   ``dict``
         """
         data = urlencode(request_body)
-        now = self._now()
         try:
             response = self.request('/o/oauth2/token', method='POST',
                                     data=data)
         except AttributeError:
             raise GoogleAuthError('Invalid authorization response, please '
-                                  'check your credentials.')
+                                  'check your credentials and time drift.')
         token_info = response.object
         if 'expires_in' in token_info:
-            expire_time = now + datetime.timedelta(
+            expire_time = _utcnow() + datetime.timedelta(
                 seconds=token_info['expires_in'])
-            token_info['expire_time'] = expire_time.strftime(TIMESTAMP_FORMAT)
+            token_info['expire_time'] = _utc_timestamp(expire_time)
         return token_info
 
     def refresh_token(self, token_info):
@@ -375,6 +377,7 @@ class GoogleBaseAuthConnection(ConnectionUserAndKey):
         :return:  A dictionary containing updated token information.
         :rtype:   ``dict``
         """
+        # pylint: disable=no-member
         return self.get_new_token()
 
 
@@ -385,6 +388,8 @@ class GoogleInstalledAppAuthConnection(GoogleBaseAuthConnection):
         Give the user a URL that they can visit to authenticate and obtain a
         code.  This method will ask for that code that the user can paste in.
 
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+
         :return:  Code supplied by the user after authenticating
         :rtype:   ``str``
         """
@@ -481,10 +486,6 @@ class GoogleServiceAcctAuthConnection(GoogleBaseAuthConnection):
                 key = key['private_key']
             except ValueError:
                 key = contents
-                logger = logging.getLogger(__name__)
-                logger.warn('%s not in JSON format.  This format is '
-                            'deprecated.  Please download a JSON key '
-                            'from the Cloud Console.' % keypath)
 
         super(GoogleServiceAcctAuthConnection, self).__init__(
             user_id, key, *args, **kwargs)
@@ -526,7 +527,7 @@ class GoogleServiceAcctAuthConnection(GoogleBaseAuthConnection):
 
 class GoogleGCEServiceAcctAuthConnection(GoogleBaseAuthConnection):
     """Authentication class for self-authentication when used with a GCE
-    istance that supports serviceAccounts.
+    instance that supports serviceAccounts.
     """
     def get_new_token(self):
         """
@@ -545,12 +546,153 @@ class GoogleGCEServiceAcctAuthConnection(GoogleBaseAuthConnection):
                              "'%s'" % str(http_reason))
         token_info = json.loads(token_info)
         if 'expires_in' in token_info:
-            expire_time = self._now() + datetime.timedelta(
+            expire_time = _utcnow() + datetime.timedelta(
                 seconds=token_info['expires_in'])
-            token_info['expire_time'] = expire_time.strftime(TIMESTAMP_FORMAT)
+            token_info['expire_time'] = _utc_timestamp(expire_time)
         return token_info
 
 
+class GoogleAuthType(object):
+    """
+    SA (Service Account),
+    IA (Installed Application),
+    GCE (Auth from a GCE instance with service account enabled)
+    GCS_S3 (Cloud Storage S3 interoperability authentication)
+    """
+    SA = 'SA'
+    IA = 'IA'
+    GCE = 'GCE'
+    GCS_S3 = 'GCS_S3'
+
+    ALL_TYPES = [SA, IA, GCE, GCS_S3]
+    OAUTH2_TYPES = [SA, IA, GCE]
+
+    @classmethod
+    def guess_type(cls, user_id):
+        if cls._is_sa(user_id):
+            return cls.SA
+        elif cls._is_gce():
+            return cls.GCE
+        elif cls._is_gcs_s3(user_id):
+            return cls.GCS_S3
+        else:
+            return cls.IA
+
+    @classmethod
+    def is_oauth2(cls, auth_type):
+        return auth_type in cls.OAUTH2_TYPES
+
+    @staticmethod
+    def _is_gce():
+        """
+        Checks if we can access the GCE metadata server.
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+        """
+        http_code, http_reason, body = _get_gce_metadata()
+        if http_code == httplib.OK and body:
+            return True
+        return False
+
+    @staticmethod
+    def _is_gcs_s3(user_id):
+        """
+        Checks S3 key format: 20 alphanumeric chars starting with GOOG.
+        """
+        return len(user_id) == 20 and user_id.startswith('GOOG')
+
+    @staticmethod
+    def _is_sa(user_id):
+        return user_id.endswith('.gserviceaccount.com')
+
+
+class GoogleOAuth2Credential(object):
+    default_credential_file = '~/.google_libcloud_auth'
+
+    def __init__(self, user_id, key, auth_type=None, credential_file=None,
+                 scopes=None, **kwargs):
+        self.auth_type = auth_type or GoogleAuthType.guess_type(user_id)
+        if self.auth_type not in GoogleAuthType.ALL_TYPES:
+            raise GoogleAuthError('Invalid auth type: %s' % self.auth_type)
+        if not GoogleAuthType.is_oauth2(self.auth_type):
+            raise GoogleAuthError(('Auth type %s cannot be used with OAuth2' %
+                                   self.auth_type))
+        self.user_id = user_id
+        self.key = key
+
+        default_credential_file = '.'.join([self.default_credential_file,
+                                            user_id])
+        self.credential_file = credential_file or default_credential_file
+        # Default scopes to read/write for compute, storage, and dns.
+        self.scopes = scopes or [
+            'https://www.googleapis.com/auth/compute',
+            'https://www.googleapis.com/auth/devstorage.full_control',
+            'https://www.googleapis.com/auth/ndev.clouddns.readwrite',
+        ]
+
+        self.token = self._get_token_from_file()
+
+        if self.auth_type == GoogleAuthType.GCE:
+            self.oauth2_conn = GoogleGCEServiceAcctAuthConnection(
+                self.user_id, self.scopes, **kwargs)
+        elif self.auth_type == GoogleAuthType.SA:
+            self.oauth2_conn = GoogleServiceAcctAuthConnection(
+                self.user_id, self.key, self.scopes, **kwargs)
+        elif self.auth_type == GoogleAuthType.IA:
+            self.oauth2_conn = GoogleInstalledAppAuthConnection(
+                self.user_id, self.key, self.scopes, **kwargs)
+        else:
+            raise GoogleAuthError('Invalid auth_type: %s' %
+                                  str(self.auth_type))
+
+        if self.token is None:
+            self.token = self.oauth2_conn.get_new_token()
+            self._write_token_to_file()
+
+    @property
+    def access_token(self):
+        if self.token_expire_utc_datetime < _utcnow():
+            self._refresh_token()
+        return self.token['access_token']
+
+    @property
+    def token_expire_utc_datetime(self):
+        return _from_utc_timestamp(self.token['expire_time'])
+
+    def _refresh_token(self):
+        self.token = self.oauth2_conn.refresh_token(self.token)
+        self._write_token_to_file()
+
+    def _get_token_from_file(self):
+        """
+        Read credential file and return token information.
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+
+        :return:  Token information dictionary, or None
+        :rtype:   ``dict`` or ``None``
+        """
+        token = None
+        filename = os.path.realpath(os.path.expanduser(self.credential_file))
+
+        try:
+            with open(filename, 'r') as f:
+                data = f.read()
+            token = json.loads(data)
+        except IOError:
+            pass
+        return token
+
+    def _write_token_to_file(self):
+        """
+        Write token to credential file.
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+        """
+        filename = os.path.realpath(os.path.expanduser(self.credential_file))
+        data = json.dumps(self.token)
+        with os.fdopen(os.open(filename, os.O_CREAT | os.O_WRONLY,
+                               int('600', 8)), 'w') as f:
+            f.write(data)
+
+
 class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
     """Base connection class for interacting with Google APIs."""
     driver = GoogleBaseDriver
@@ -574,13 +716,8 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
                      authentication.
         :type   key: ``str``
 
-        :keyword  auth_type: Accepted values are "SA" or "IA" or "GCE"
-                             ("Service Account" or "Installed Application" or
-                             "GCE" if libcloud is being used on a GCE instance
-                             with service account enabled).
-                             If not supplied, auth_type will be guessed based
-                             on value of user_id or if the code is being
-                             executed in a GCE instance.).
+        :keyword  auth_type: See GoogleAuthType class for list and description
+                             of accepted values.
                              If not supplied, auth_type will be guessed based
                              on value of user_id or if the code is running
                              on a GCE instance.
@@ -594,65 +731,21 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
                           read/write access to Compute, Storage, and DNS.
         :type     scopes: ``list``
         """
-        self.credential_file = credential_file or '~/.gce_libcloud_auth'
-
-        if auth_type is None:
-            # Try to guess.
-            if '@' in user_id:
-                auth_type = 'SA'
-            elif _is_gce():
-                auth_type = 'GCE'
-            else:
-                auth_type = 'IA'
-
-        # Default scopes to read/write for compute, storage, and dns.  Can
-        # override this when calling get_driver() or setting in secrets.py
-        self.scopes = scopes
-        if not self.scopes:
-            self.scopes = [
-                'https://www.googleapis.com/auth/compute',
-                'https://www.googleapis.com/auth/devstorage.full_control',
-                'https://www.googleapis.com/auth/ndev.clouddns.readwrite',
-            ]
-        self.token_info = self._get_token_info_from_file()
-
-        if auth_type == 'GCE':
-            self.auth_conn = GoogleGCEServiceAcctAuthConnection(
-                user_id, self.scopes, **kwargs)
-        elif auth_type == 'SA':
-            if '@' not in user_id:
-                raise GoogleAuthError('Service Account auth requires a '
-                                      'valid email address')
-            self.auth_conn = GoogleServiceAcctAuthConnection(
-                user_id, key, self.scopes, **kwargs)
-        elif auth_type == 'IA':
-            self.auth_conn = GoogleInstalledAppAuthConnection(
-                user_id, key, self.scopes, **kwargs)
-        else:
-            raise GoogleAuthError('Invalid auth_type: %s' % str(auth_type))
-
-        if self.token_info is None:
-            self.token_info = self.auth_conn.get_new_token()
-            self._write_token_info_to_file()
-
-        self.token_expire_time = datetime.datetime.strptime(
-            self.token_info['expire_time'], TIMESTAMP_FORMAT)
-
         super(GoogleBaseConnection, self).__init__(user_id, key, **kwargs)
 
+        self.oauth2_credential = GoogleOAuth2Credential(
+            user_id, key, auth_type, credential_file, scopes, **kwargs)
+
         python_ver = '%s.%s.%s' % (sys.version_info[0], sys.version_info[1],
                                    sys.version_info[2])
         ver_platform = 'Python %s/%s' % (python_ver, sys.platform)
         self.user_agent_append(ver_platform)
 
-    def _now(self):
-        return datetime.datetime.utcnow()
-
     def add_default_headers(self, headers):
         """
         @inherits: :class:`Connection.add_default_headers`
         """
-        headers['Content-Type'] = "application/json"
+        headers['Content-Type'] = 'application/json'
         headers['Host'] = self.host
         return headers
 
@@ -663,15 +756,8 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
 
         @inherits: :class:`Connection.pre_connect_hook`
         """
-        now = self._now()
-        if self.token_expire_time < now:
-            self.token_info = self.auth_conn.refresh_token(self.token_info)
-            self.token_expire_time = datetime.datetime.strptime(
-                self.token_info['expire_time'], TIMESTAMP_FORMAT)
-            self._write_token_info_to_file()
-        headers['Authorization'] = 'Bearer %s' % (
-            self.token_info['access_token'])
-
+        headers['Authorization'] = ('Bearer ' +
+                                    self.oauth2_credential.access_token)
         return params, headers
 
     def encode_data(self, data):
@@ -699,33 +785,6 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
         # One more time, then give up.
         return super(GoogleBaseConnection, self).request(*args, **kwargs)
 
-    def _get_token_info_from_file(self):
-        """
-        Read credential file and return token information.
-
-        :return:  Token information dictionary, or None
-        :rtype:   ``dict`` or ``None``
-        """
-        token_info = None
-        filename = os.path.realpath(os.path.expanduser(self.credential_file))
-
-        try:
-            with open(filename, 'r') as f:
-                data = f.read()
-            token_info = json.loads(data)
-        except IOError:
-            pass
-        return token_info
-
-    def _write_token_info_to_file(self):
-        """
-        Write token_info to credential file.
-        """
-        filename = os.path.realpath(os.path.expanduser(self.credential_file))
-        data = json.dumps(self.token_info)
-        with open(filename, 'w') as f:
-            f.write(data)
-
     def has_completed(self, response):
         """
         Determine if operation has completed based on response.
