@@ -42,12 +42,12 @@ class ProfitBricksTests(unittest.TestCase):
         node = nodes[0]
         self.assertEqual(node.id, "c8e57d7b-e731-46ad-a913-1828c0562246")
         self.assertEqual(node.name, "server001")
-        self.assertEqual(node.state, 0)
+        self.assertEqual(node.state, NodeState.RUNNING)
         self.assertEqual(node.public_ips, ['162.254.25.197'])
         self.assertEqual(node.private_ips, ['10.10.108.12', '10.13.198.11'])
         self.assertEqual(node.extra['datacenter_id'], "e1e8ec0d-b47f-4d39-a91b-6e885483c899")
         self.assertEqual(node.extra['datacenter_version'], "5")
-        self.assertEqual(node.extra['provisioning_state'], 0)
+        self.assertEqual(node.extra['provisioning_state'], NodeState.RUNNING)
         self.assertEqual(node.extra['creation_time'], "2014-07-14T20:52:20.839Z")
         self.assertEqual(node.extra['last_modification_time'], "2014-07-14T22:11:09.324Z")
         self.assertEqual(node.extra['os_type'], "LINUX")
@@ -121,7 +121,7 @@ class ProfitBricksTests(unittest.TestCase):
         self.assertEqual(volume.name, "storage001")
         self.assertEqual(volume.size, 50)
         self.assertEqual(volume.extra['server_id'], "ebee7d83-912b-42f1-9b62-b953351a7e29")
-        self.assertEqual(volume.extra['provisioning_state'], 0)
+        self.assertEqual(volume.extra['provisioning_state'], NodeState.RUNNING)
         self.assertEqual(volume.extra['creation_time'], "2014-07-15T03:19:38.252Z")
         self.assertEqual(volume.extra['last_modification_time'], "2014-07-15T03:28:58.724Z")
         self.assertEqual(volume.extra['image_id'], "d2f627c4-0289-11e4-9f63-52540066fee9")
@@ -331,7 +331,7 @@ class ProfitBricksTests(unittest.TestCase):
         interface = interfaces[0]
         self.assertEqual(interface.id, "6b38a4f3-b851-4614-9e3a-5ddff4727727")
         self.assertEqual(interface.name, "StackPointCloud")
-        self.assertEqual(interface.state, 0)
+        self.assertEqual(interface.state, NodeState.RUNNING)
         self.assertEqual(interface.extra['server_id'], "234f0cf9-1efc-4ade-b829-036456584116")
         self.assertEqual(interface.extra['lan_id'], '3')
         self.assertEqual(interface.extra['internet_access'], 'false')
@@ -376,7 +376,7 @@ class ProfitBricksTests(unittest.TestCase):
 
         self.assertEqual(describe.id, "f1c7a244-2fa6-44ee-8fb6-871f337683a3")
         self.assertEqual(describe.name, None)
-        self.assertEqual(describe.state, 0)
+        self.assertEqual(describe.state, NodeState.RUNNING)
         self.assertEqual(describe.extra['datacenter_id'], "a3a2e730-0dc3-47e6-bac6-4c056d5e2aee")
         self.assertEqual(describe.extra['datacenter_version'], "6")
         self.assertEqual(describe.extra['server_id'], "c09f4f31-336c-4ad2-9ec7-591778513408")
